#!/bin/sh
#
# Copyright (C) 2022 by Katsumi Nuruki, all reghts recerved.
#
#ident "@(#)sardo.sh      1.1e   2022.06.12 Katsumi.Nuruki"
#
# gather the system performance information for Linux.
#
#

#
# External environment variables:
#

# variable description:
# LOGDIR        This environment variable determines where sardo.sh data will
#               be gathered. $LOGDIR also use for the other sardo.sh functions.
#               The default value of $LOGDIR is "/var/tmp".
# SARDO         This environment variable is used for an instance identifier.
#               When the SARDO environment variable isn't defined, it will be
#               assumed three digits "001" which is defined same as "1".
#               Following sample is consequent of sardo directory name, if the
#               SARDO  environment variable is set to "2".
#
#                       "${LOGDIR}/sarstats_${HOSTNAME}_002"
#
# HOSTNAME      This environment variable is used for the hostname identifier.
#               When the HOSTNAME environment variable isn't defined, it will
#               be assumed `uname -n` output of this system.
#
# PACCTMODE     This environment variable is used for process account handling.
#               Default value is "RUN". If disable process account handling,
#               set to "DONTRUN".
#

#
# Updata information.
# sardo.sh was created as first version.                     2004.03.18 Rev.1.0a
# COR-001 The function name bug fix.                         2004.05.11 Rev.1.0b
# COR-002 Change eventcheck routine.                         2004.07.04 Rev.1.0c
# COR-003 Prevent time delaying.                             2004.07.04 Rev.1.0c
# COR-004 Add ifconfig -a command.                           2004.10.18 Rev.1.0d
# COR-005 Change for sysstat 6.0.2 & 8.1.5.                  2010.12.03 Rev.1.0g
# COR-006 Fix invalid time interval.                         2010.05.10 Rev.1.0e
# COR-007 Add param.h gathering for SUSE Linux.              2010.05.10 Rev.1.0e
# COR-008 Add mpstat -P n function for SMP system.           2010.05.17 Rev.1.0e
# COR-009 Check sadc PID is integer expression.              2010.05.19 Rev.1.0e
# COR-010 Add uname -p command for misc info.                2010.05.22 Rev.1.0e
# COR-011 Gather /proc/meminfo information.                  2010.05.24 Rev.1.0e
# COR-012 Check command path(Ubuntu,Debian etc).             2010.05.31 Rev.1.0e
# COR-013 Check /proc/ioports(Gentoo) if error.              2010.06.03 Rev.1.0e
# COR-014 Check accton version for Turbo Linux.              2010.06.03 Rev.1.0e
# COR-015 Change for sysstat version 4.0.1.                  2010.11.16 Rev.1.0g
# COR-016 Change ps -efly to ps -eflyww.                     2010.12.03 Rev.1.0g
# COR-017 Change for sysstat 6.0.2.                          2010.12.03 Rev.1.0g
# COR-018 Fix ps file name problem in spexpmap.              2010.12.17 Rev.1.0h
# COR-019 Correct package version check.                     2015.08.29 Rev.1.0k
# COR-020 Change accton 6.6.1 command argument.              2015.08.30 Rev.1.0k
# COR-021 Change PACCTMODE environment variable.             2018.11.05 Rev.1.1a
# COR-022 Change sar command permission.                     2018.11.05 Rev.1.1a
# COR-023 Change register file permission.                   2019.01.21 Rev.1.1a
# COR-024 Change sysstat version checking.                   2019.07.19 Rev.1.1b
# COR-025 Add ACCTMODE change status.                        2019.07.22 Rev.1.1b
# COR-026 Change accton 6.6.3 command argument.              2020.09.30 Rev.1.1c
#

# Tools Level
sysstat="4.0.1"							        #COR-015
netstat="1.42"
accton="6.3.2"
acctlv="6.5.3"								#COR-014
acctcent7="6.6.1"							#COR-020
acctcent8="6.6.3"							#COR-026
acctlatest="6.6.4"							#COR-026
#

BIN=/bin
ETCDIR=/etc
OSNAME=Linux
PROC=/proc
USRBIN=/usr/bin
USRLIBSA=/usr/lib/sa
USRINCLUDEASM=/usr/include/asm
USRSBIN=/usr/sbin
SBIN=/sbin								# COR-004

BASEINTERVAL=5
ONEDAYSECONDS=86400

MISCCMD="\
$USRBIN/du -k
$BIN/uname -p
$BIN/uname -r
$USRBIN/hostid"
# COR-010 add uname -p command.

# COR-004 Add ifconfig -a command.
# COR-011 Add meminfo.
# COR-016 Chg ps -efly to ps -eflyww.
ONETIMECMD="\
$BIN/df -k
$SBIN/ifconfig -a
$USRBIN/ipcs -a
$BIN/netstat -s
$BIN/ps -eflyww
meminfo
pmap -x"

DURCMD="\
$USRBIN/iostat -x
$USRBIN/vmstat
$USRBIN/mpstat"

NDCMD="\
$BIN/ps -efl
$BIN/netstat -i
$BIN/netstat -s
$BIN/netstat -an"

SADCCMD="\
$USRBIN/sar"
#$USRLIBSA/sadc"							# COR-005

GATHERFILES="\
$PROC/apm
$PROC/cpuinfo
$PROC/devices
$PROC/dma
$PROC/filesystems
$PROC/interrupts
$PROC/iomem
$PROC/ioports
$PROC/ksyms
$PROC/loadavg
$PROC/mdstat
$PROC/meminfo
$PROC/misc
$PROC/modules
$PROC/mounts
$PROC/mtrr
$PROC/net
$PROC/partitions
$PROC/pci
$PROC/slabinfo
$PROC/stat
$PROC/swaps
$PROC/sys
$PROC/sysvipc
$PROC/tty
$PROC/uptime
$PROC/version
$ETCDIR/hosts
$ETCDIR/system
$ETCDIR/fstab
$USRBIN/sar
$USRINCLUDEASM/page.h
$USRINCLUDEASM/param.h"
# param.h added by COR-007.

EVENTCHECK=sardo.event							# COR-002

intervaltimelist="\
3600
1800
900
600
60
30
10
5"

samplingtimelist="\
300
600
1800
3600
7200
10800
14400
18000
21600
25200
28800
32400
36000
39600
43200
46800
50400
54000
57600
61200
64800
68400
72000
75600
79200
82800
86400"

#
# Check the level number of statistics tools.
#
CheckToolsLevel() {
#
if [ $FUNCTION = "auto" ] || [ $FUNCTION = "setup" ]; then
	if [ ! -d ${SARDODIR}/misc ]; then
		$BIN/mkdir ${SARDODIR}/misc
		if [ $? -ne 0 ]; then
			LogRecord "ERROR: making misc directory was failed."
			ClearfAndExit 1
		fi
	fi
	cd ${SARDODIR}/misc
	if [ ! -f ${SARDODIR}/misc/version_info ]; then
		cd ${SARDODIR}/misc
		$BIN/uname -r > version_info
		### sar command.
		if [ ! -e $USRBIN/sar ]; then
			$BIN/echo "ERROR: sar command not found."
			ClearfAndExit 1
		fi
		$USRBIN/sar -V 1> $WORK 2>&1
		SYSSTAT=`$AWK 'BEGIN {	result="NG"
						rlsid="'"$sysstat"'"
					}
		{
		if ( $1 == "sysstat" ) {					# COR-024
			printf"%s %s %s\n",$1,$2,$3 >>"version_info"
			n=split($3,array,".")
			for ( i = 0; i <= n; i++ )
				sysid=int(sprintf("%02d%02d",sysid,array[i]))	# COR-019
			n=split(rlsid,array,".")
			for ( i = 0; i <= n; i++ )
				chkid=int(sprintf("%02d%02d",chkid,array[i]))	# COR-019
			if ( sysid >= chkid ) {
				result="OK"
				exit
				}
			}
		} END {	printf"%s\n",result }' $WORK`
		if [ $SYSSTAT = "NG" ]; then
			$BIN/echo "WARN:  sysstat version is lower than ${sysstat}."
		fi
		#
		### netstat command.
		if [ ! -e $BIN/netstat ]; then
			$BIN/echo "ERROR: netstat command not found."
			ClearfAndExit 1
		fi
		$BIN/netstat -V 1>$WORK 2>&1
		NETSTAT=`$AWK 'BEGIN {	result="NG"
						rlsid="'"$netstat"'"
					}
		{
		# COR-019 start.
		if (( $1 == "net-tools" ) && ( NF == 2 )) {
			printf"%s %s\n",$1,$2 >>"version_info"
			split($2,array,"-")
			split(array[1],wstr,".")
			sysid = int(sprintf("%02d%02d",wstr[0],wstr[1]))
			if ( sysid >= chkid ) {
				result="OK"
				exit
			}
		}
		# COR-019 end.
		if (( $1 == "netstat" ) && ( NF == 3 )) {
			printf"%s %s %s\n",$1,$2,$3 >>"version_info"
			n=split($2,array,".")
			for ( i = 0; i <= n; i++ )
				sysid=int(sprintf("%02d%02d",sysid,array[i]))	# COR-019
			n=split(rlsid,array,".")
			for ( i = 0; i <= n; i++ )
				chkid=int(sprintf("%02d02%d",chkid,array[i]))	# COR-019
			if ( sysid >= chkid ) {
				result="OK"
				exit
				}
			}
		} END {	printf"%s\n",result }' $WORK`
		if [ $NETSTAT = "NG" ]; then
			$BIN/echo "WARN:  netstat version is lower than ${netstat}."
		fi
		#
		### accton command.
		# COR-021 start.
		if [ "${PACCTMODE}_x" = "_x" ]; then
			if [ ! -e $USRSBIN/accton ]; then
				$BIN/echo "NOTE:  accton command not found."
				PACCTMODE="DONTRUN"
			else
				PACCTMODE="RUN"
			fi
		fi
		# COR-021 end.
		if [ $PACCTMODE = "RUN" ]; then
			$USRSBIN/accton -V 1>$WORK 2>&1
			ACTSTAT=`$AWK 'BEGIN {	result="NG"
						rlsid="'"$accton"'" }
			{
			n=split($1,command,"/")
			if (( command[n] == "accton:" ) && ( $3 == "Accounting" )) {
				printf"%s\n",$0 >>"version_info"
				split($6,release,")")
				n=split(release[1],array,".")
				for ( i = 0; i <= n; i++ )
					sysid=sprintf("%d%d",sysid,array[i])
				n=split(rlsid,array,".")
				for ( i = 0; i <= n; i++ )
					chkid=sprintf("%d%d",chkid,array[i])
				if ( sysid >= chkid ) {
					result="OK"
					exit
					}
				}
			} END {	printf"%s\n",result }' $WORK`
			if [ $ACTSTAT = "NG" ]; then
				$BIN/echo "WARN:  accton version is lower than ${accton}."
			fi
		fi # COR-021
	fi
fi
}


#
# Log records and display.
#
LogRecord() {
arg1="$1"
DATE=`$BIN/date '+%y%m%d%H%M%S'`
function=`$BIN/echo $FUNCTION |
$AWK '{ printf"%-6s\n",substr(toupper($1),1,6) }'`
$BIN/echo "$DATE $function :`$BIN/echo "$arg1"`" >> $LOGFILE
if [ $BATCHMODE = "NO" ]; then
	$BIN/echo "$arg1"
fi
}


#
# Set BATCHMODE environment variable.
#
SetBatchMode() {
if [ "${BATCHMODE}_x" = "_x" ]; then
	BATCHMODE=NO
else
	if [ $BATCHMODE != "YES" ] && [ $BATCHMODE != "NO" ]; then
		BATCHMODE=NO
	fi
fi
}


#
# Set the instance id and the other environment variables.
#
SetInstance() {
SET="NO"
if [ "${SARDO}_x" = "_x" ]; then
	INSTANCE="001"
else
	INSTANCE=`$BIN/echo $SARDO | $AWK '{ printf"%03d\n",$1 }'`
fi
if [ "${HOSTNAME}_x" = "_x" ]; then
	HOSTNAME=`$BIN/uname -n`
fi
SARDODIR=$LOGDIR/sarstats_${HOSTNAME}_${INSTANCE}
LOGFILE=$SARDODIR/sardo.log
PARAMETER=$SARDODIR/sardo.param
TEMPFILE=$SARDODIR/tempfile$$
WORKFILE=$SARDODIR/workfile$$
WORK=$LOGDIR/workfile$$
SARDO=$INSTANCE
export SARDO LOGDIR SARDODIR LOGFILE PARAMETER
}


#
# Check root uid.
#
UidCheck() {
UUID=`$USRBIN/id |
$AWK '{	split($1,array,"=")
		split(array[2],id,"(")
		if (( id[1] == 0 ) && ( substr(id[2],1,4) == "root" ))
			result="OK"
		else
			result="NG"
		}
END {	print result }'`
if [ $UUID != "OK" ]; then
	$BIN/echo "ERROR: uid is not root."
	ClearfAndExit 1
fi
}


#
# get the command path instead of which(1) command.
#
whichcmd () {
mod=$1
if [ "${mod}_x" = "_x" ]; then
	LogRecord "ERROR: whichcmd argument missing."
	ClearfAndExit 1
fi
COMMANDPATH=NOTFOUND
found=NO
while [ $found = "NO" ]
do
	for i in `$BIN/echo $PATH | $AWK '{
		n = split($0,array,":")
		for ( i = 1 ; i <= n ; i++ )
			printf"%s\n",array[i]
		printf"\n" }'`
	do
		if [ -x ${i}/${mod} ]  && [ "${i}" != "." ]; then
			COMMANDPATH=${i}/${mod}
			found=YES
			break
		fi
	done
	found=$COMMANDPATH
done
}


#
# Check and make sardo directory.
#
SarDoDirectory() {
if [ ! -d $SARDODIR ]; then
	if [ $FUNCTION != "unpack" ] && [ $FUNCTION != "estimate" ]; then
		$BIN/mkdir $SARDODIR
		if [ $? -ne 0 ]; then
			LogRecord "ERROR: can't make sardo directory."
			ClearfAndExit 1
		fi
	fi
fi
}


#
# Print "."(dot) without LF.
#
PrintDot() {
if [ $BATCHMODE = "NO" ]; then
	$BIN/echo -e ".\c"
fi
}


#
# Feed one line.
#
BlankLine() {
if [ $BATCHMODE = "NO" ]; then
	$BIN/echo ""
fi
}


#
# Setup the parameter file.
#
spsetup() {
if [ -f ${SARDODIR}/pidlist ]; then
	LogRecord "ERROR: instance $INSTANCE has already been started."
	ClearfAndExit 1
fi
case $numberofargs in
4)	SetTimeParameters $ARG2 $ARG3 $ARG4
	$BIN/echo "$INTERVAL $STIME $ETIME" > $PARAMETER
	RETVAL=$?					;;
*)	RETVAL=1					;;
esac
if [ $RETVAL -ne 0 ]; then
	LogRecord "ERROR: spsetup parameter setting was failed."
	ClearfAndExit 1
fi
LogRecord "INFO:  spsetup was successfully completed."
}


#
# Execute one time commands.
#
DoCommands() {
ID=$1
if [ ! -d $SARDODIR/${ID} ]; then
	$BIN/mkdir $SARDODIR/${ID}
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: can't make $SARDODIR/${ID} direcotry."
		ClearfAndExit 1
	fi
fi
CMDCOUNT=`$USRBIN/wc -l $TEMPFILE | $AWK '{ print $1 }'`
N=1
while [ $N -le $CMDCOUNT ]
do
	CMD=`$BIN/cat $TEMPFILE |
	$AWK '{	if ( NR == '"$N"' )
				print $0 }'`
	BASENAME=`$BIN/echo "$CMD" | $AWK '{ print $1 }'`
	DIRNAME=`$USRBIN/dirname $BASENAME`
	BASENAME=`$BASENAMECMD $BASENAME`
	SUFFICS=`$BIN/echo "$CMD" |
	$AWK '{ if ( NF == 2 ) {
				split($2,array,"-")
				name=array[2]
				}
			else
				name=""
			}
	END { print name }'`
	DATE=`$BIN/date '+%H:%M:%S'`
	OFILE=`$BIN/echo $SARDODIR/${ID}/${BASENAME}_${SUFFICS}_info |
	$BIN/sed 's/__/_/g'`
	$BIN/echo "${BASENAME}_${SUFFICS} $DATE" > $OFILE
	if [ $BASENAME = "pmap" ]; then
		for pid in `$BIN/ps -efl |
		$AWK '{	if ( $4 != "PID" )
					print $4 }'`
		do
			$BIN/echo "bgn: $pid"
			$BIN/cat ${PROC}/${pid}/maps 2>/dev/null
			$BIN/echo "end: $pid"
		done >> $OFILE
		RETVAL=0
	else
		# COR-011 start.
		if [ $BASENAME = "meminfo" ]; then
			$BIN/cat $PROC/meminfo >> $OFILE
		else
		# COR-011 end.
		if [ ! -x $DIRNAME/$BASENAME ]; then
			LogRecord "ERROR: $DIRNAME/$BASENAME isn't executable."
		else
			case $BASENAME in
			*)	$CMD >> $OFILE
				RETVAL=$?		;;
			esac
		    fi
		fi
	fi
	if [ $RETVAL -ne 0 ]; then
		# COR-012 start.
		if [ $BASENAME = "netstat" ] && [ $KERNEL = "Debian" ]; then
			LogRecord "INFO:  $BASENAME -${SUFFICS} was successfully completed."
		else
			LogRecord "ERROR: $BASENAME -${SUFFICS} return value wasn't 0"
		fi
		# COR-012 end.
	else
		LogRecord "INFO:  $BASENAME -${SUFFICS} was successfully completed."
	fi
	N=`$USRBIN/expr $N + 1`
done
}


#
# Execute the duration type commands.
#
DoDurationCmds() {
ID=$1
if [ ! -d $SARDODIR/${ID} ]; then
	$BIN/mkdir $SARDODIR/${ID}
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: can't make $SARDODIR/${ID} direcotry."
		ClearfAndExit 1
	fi
fi
$AWK '{ if ( NF != 0 ) print $0 }' $TEMPFILE > $WORKFILE
$BIN/mv $WORKFILE $TEMPFILE
DOCOUNT=`$USRBIN/expr $COUNT + 1`
CMDCOUNT=`$USRBIN/wc -l $TEMPFILE | $AWK '{ print $1 }'`
N=1
while [ $N -le $CMDCOUNT ]
do
	CMD=`$BIN/cat $TEMPFILE |
	$AWK '{	if ( NR == '"$N"' )
				print $0 }'`
	IF=`$BIN/echo "$CMD" | $AWK '{ print $3 }'`
	BASENAME=`$BIN/echo "$CMD" | $AWK '{ print $1 }'`
	DIRNAME=`$USRBIN/dirname $BASENAME`
	BASENAME=`$BASENAMECMD $BASENAME`
	SUFFICS=`$BIN/echo "$CMD" |
	$AWK '{ if ( NF == 2 ) {
				split($2,array,"-")
				name=array[2]
				}
			else
				name=""
			}
	END { print name }'`
	OFILE=`$BIN/echo $SARDODIR/${ID}/${BASENAME}_${SUFFICS}_info |
	$BIN/sed 's/__/_/g'`
	if [ ! -x $DIRNAME/$BASENAME ]; then
		LogRecord "ERROR: $DIRNAME/$BASENAME isn't executable."
	else
		case $BASENAME in
		# COR-008 start.
		mpstat)		CPUNUMBER=`$BIN/cat $PROC/cpuinfo |
				$AWK 'BEGIN { cpucount=0 }
				{	if ( $1 == "processor" )
						cpucount = cpucount + 1
				} END {	print cpucount-1 }'`
				$CMD $INTERVAL $DOCOUNT > $SARDODIR/${ID}/${BASENAME}_all_info &
				PID=$!
				RETVAL=$?
				if [ $RETVAL -ne 0 ]; then
					LogRecord "ERROR: $BASENAME return value wasn't 0"
				else
					LogRecord "INFO:  $BASENAME was successfully scheduled."
				fi
				$BIN/echo $PID >> $SARDODIR/pidlist
				if [ $CPUNUMBER -gt 0 ]; then
					while [ $CPUNUMBER -ge 0 ]
					do
						$CMD -P $CPUNUMBER $INTERVAL $DOCOUNT > $SARDODIR/${ID}/${BASENAME}_${CPUNUMBER}_info &
						PID=$!
						RETVAL=$?
						if [ $RETVAL -ne 0 ]; then
							LogRecord "ERROR: $BASENAME -P ${CPUNUMBER} return value wasn't 0"
						else
							LogRecord "INFO:  $BASENAME -P ${CPUNUMBER} was successfully scheduled."
						fi
						$BIN/echo $PID >> $SARDODIR/pidlist
						CPUNUMBER=`$USRBIN/expr $CPUNUMBER - 1`
					done
				fi					;;
		# COR-008 end.
		*)		$CMD $INTERVAL $DOCOUNT > $OFILE &
				PID=$!
				RETVAL=$?	
				if [ $RETVAL -ne 0 ]; then
					LogRecord "ERROR: $BASENAME -${SUFFICS} return value wasn't 0"
				else
					LogRecord "INFO:  $BASENAME -${SUFFICS} was successfully scheduled."
				fi
				$BIN/echo $PID >> $SARDODIR/pidlist	;;
		esac
	fi
	N=`$USRBIN/expr $N + 1`
done
}


#
# Execute the non duration type commands.
#
DoNonDurationCmds() {
ID=$1
if [ ! -d $SARDODIR/${ID} ]; then
	$BIN/mkdir $SARDODIR/${ID}
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: can't make $SARDODIR/${ID} direcotry."
		ClearfAndExit 1
	fi
fi
$BIN/echo "`$BIN/date '+%H:%M:%S'`" > $SARDODIR/go
CMDCOUNT=`$USRBIN/wc -l $TEMPFILE | $AWK '{ print $1 }'`
N=1
while [ $N -le $CMDCOUNT ]
do
	CMD=`$BIN/cat $TEMPFILE |
	$AWK '{	if ( NR == '"$N"' )
				print $0 }'`
	BASENAME=`$BIN/echo "$CMD" | $AWK '{ print $1 }'`
	DIRNAME=`$USRBIN/dirname $BASENAME`
	BASENAME=`$BASENAMECMD $BASENAME`
	SUFFICS=`$BIN/echo "$CMD" |
	$AWK '{ if ( NF == 2 ) {
				split($2,array,"-")
				name=array[2]
				}
			else
				name=""
			}
	END { print name }'`
	DATE=`$BIN/date '+%H:%M:%S'`
	OFILE=`$BIN/echo $SARDODIR/${ID}/${BASENAME}_${SUFFICS}_info |
	$BIN/sed 's/__/_/g'`
	if [ ! -x $DIRNAME/$BASENAME ]; then
		LogRecord "ERROR: $DIRNAME/$BASENAME isn't executable."
	else
		while [ -f $SARDODIR/go ]
		do
			$BIN/echo "${BASENAME}_${SUFFICS} `$BIN/date '+%H:%M:%S'`" >> $OFILE
			$CMD >> $OFILE &				# COR-003
			$BIN/sleep $INTERVAL
		done &
		PID=$!
		RETVAL=$?
		if [ $RETVAL -ne 0 ]; then
			LogRecord "ERROR: $BASENAME -${SUFFICS} return value wasn't 0"
		else
			LogRecord "INFO:  $BASENAME -${SUFFICS} was successfully scheduled."
		fi
		$BIN/echo $PID >> $SARDODIR/pidlist
	fi
	N=`$USRBIN/expr $N + 1`
done
}


#
# Execute the sadc commands.
#
DoSaDcCommand() {
ID=$1
if [ ! -d $SARDODIR/${ID} ]; then
	$BIN/mkdir $SARDODIR/${ID}
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: can't make $SARDODIR/${ID} direcotry."
		ClearfAndExit 1
	fi
fi
CMD=`$BIN/cat $TEMPFILE`
if [ ! -x $CMD ]; then
	LogRecord "ERROR: $CMD isn't executable."
	ClearfAndExit 1							# COR-001
fi
SADCCOUNT=`$BIN/echo $COUNT |
$AWK '{	count=$1*'"$INTERVAL"'/'"$BASEINTERVAL"'
		count++ }
END { printf("%.0f\n",count) }'`					# COR-006
if [ -f $SARDODIR/${ID}/sadc_info ]; then
	$BIN/rm $SARDODIR/${ID}/sadc_info
fi
# COR-005 $CMD $BASEINTERVAL $SADCCOUNT $SARDODIR/${ID}/sadc_info &
# COR-017 start.
sysstatversion=`$AWK '{ if ( $1 == "sysstat" ) print $3 }' $SARDODIR/misc/version_info`		# COR-024
if [ $sysstatversion = "6.0.2" ]; then
	$USRLIBSA/sadc $BASEINTERVAL $SADCCOUNT $SARDODIR/${ID}/sadc_info &
else
	$USRBIN/sar -A -o $SARDODIR/${ID}/sadc_info $BASEINTERVAL $SADCCOUNT >/dev/null &	# COR-005
fi
# COR-017 end.

PID=$!
RETVAL=$?
if [ $RETVAL -ne 0 ]; then
	LogRecord "ERROR: sadc command return value wasn't 0"
else
	LogRecord "INFO:  sadc command was successfully scheduled."
fi
# COR-005 start.
$BIN/echo $PID >> $SARDODIR/pidlist
PID=`$BIN/ps -efl |
$AWK '{
	if ($5 == "'$PID'") {
		print $4
		exit
	}
}'`
# COR-005 end.
$BIN/echo $PID >> $SARDODIR/pidlist
}


#
# Check process account is running or not.
#
CheckPacct() {
#PACCTMODE=RUN							# COR-021
if [ -f $ACCTLOCK ]; then
	PACCTMODE=DONTRUN
	LogRecord "NOTE:  PACCTMODE to DONTRUN due to $ACCTLOCK file."	# COR-025
fi
}


#
# Clear files and terminating with arg1.
#
ClearfAndExit() {
RETVAL=$1
for i in $TEMPFILE $WORKFILE $WORK
do
	if [ -f $i ]; then
		$BIN/rm $i
	fi
done
exit $RETVAL
}


#
# Convert the time string to the integer in seconds.
#
S2I() {
TIMEVALUE=`$BIN/echo $1 |
$AWK '{ n=split($1,array,":") }
END {	if ( n > 1 ) {
		if (( array[1] < 24 ) && ( array[1] >= 0 ))
			hsec=array[1]*3600
		else {
			if ( array[1] == 24 ) {
				printf"%d\n",array[1]*3600
				exit
			}
			printf"INVALID\n"
			exit
		}
		if (( array[2] < 60 ) && ( array[2] >= 0 ))
			msec=array[2]*60
		else {
			printf"INVALID\n"
			exit
		}
		if ( n == 3 ) {
			if (( array[3] < 60 ) && ( array[3] >= 0 ))
				sec=array[3]
			else {
				printf"INVALID\n"
				exit
			}
		}
		else
			sec=0
		printf"%d\n",hsec+msec+sec
		}
	else
		printf"INVALID\n" }'`
if [ $TIMEVALUE = "INVALID" ]; then
	LogRecord "ERROR: the time format invalid."
	ClearfAndExit 1
fi
}


#
# Convert the integer second to the time string.
#
I2S() {
TIMESTRING=`$BIN/echo $1 |
$AWK '{	h=$1/3600
		r=$1%3600
		m=r/60
		s=r%60
		if ( h >= 24 )
			h=h-24
		printf"%02d:%02d:%02d\n",h,m,s }'`
}


#
# Set the times environment variables.
#
SetTimeParameters() {
arg1=$1
arg2=$2
arg3=$3
#
# $arg1         : sampling interval xx in seconds, xxmin, or xxsec.
# $arg2         : start time now or HH:MM[:SS] format.
# $arg3         : end time xxmin, xxhour, or HH:MM[:SS] format.
#
# This function sets times environment variables as below.
# $SAMPLING     : sampling mode. INTHEDAY   = $arg2 < $arg3
#                                OVERTHEDAY = $arg2 > $arg3
# $INTERVAL	: sampling interval in seconds.
# $COUNT	: sampling count.
# $ctime        : current time in seconds.
# $CTIME        : current time with HH:MM:SS format.
# $stime	: start time in seconds.
# $STIME	: start time with HH:MM:SS format.
# $etime	: end time in seconds.
# $ETIME	: end time with HH:MM:SS format.
#
#
INTERVAL=`$BIN/echo $arg1 |
$AWK '{
if ( substr($1,length($1)-3+1,3) == "min" ) {
	split($1,array,"min")
	if (( array[1] <= 60 ) && ( array[1] >= 1 ))
		n=array[1]*60
	else
		n="NG"
	}
else
	if ( substr($1,length($1)-3+1,3) == "sec" ) {
		split($1,array,"sec")
		if (( array[1] <= 3600 ) && ( array[1] >= 5))
			n=array[1]
		else
			n="NG"
		}
	else
		if (( $1 <= 3600 ) && ( $1 >= 5 ))
			n=$1
		else
			n="NG"
	}
END { printf"%d\n",n }'`
if [ $INTERVAL = "NG" ]; then
	LogRecord "ERROR: the sampling interval invalid."
	ClearfAndExit 1
fi
CTIME=`$BIN/date '+%H:%M:%S'`
S2I $CTIME
ctime=$TIMEVALUE
if [ $arg2 = "now" ]; then
	STIME=$CTIME
else
	S2I $arg2
	I2S $TIMEVALUE
	STIME=$TIMESTRING
fi
S2I $STIME
stime=$TIMEVALUE
$BIN/echo $arg3 | $BIN/grep ":" 1>/dev/null
if [ $? -eq 0 ]; then
	S2I $arg3
	etime=$TIMEVALUE
else
	RETVAL=`$BIN/echo $arg3 |
	$AWK '{
	if ( substr($1,length($1)-3+1,3) == "min" ) {
		split($1,array,"min")
		if (( array[1] <= 1440 ) && ( array[1] >=1 ))
			n=array[1]*60
		else
			n="NG"
		}
	else
		if ( substr($1,length($1)-4+1,4 ) == "hour" ) {
			split($1,array,"hour")
			if (( array[1] <= 24 ) && ( array[1] >=1 ))
				n=array[1]*3600
			else
				n="NG"
			}
		else
			n=$1
		}
	END { printf"%s\n",n }'`
	if [ $RETVAL = "NG" ]; then
		LogRecord "ERROR: the end parameter is invalid."
		ClearfAndExit 1
	fi
	etime=`$USRBIN/expr $RETVAL + $stime`
fi
TIMEDIFF=`$USRBIN/expr $etime - $stime`
if [ $TIMEDIFF -eq $ONEDAYSECONDS ]; then
	etime=`$USRBIN/expr $etime - 1`
fi
I2S $etime
ETIME=$TIMESTRING
if [ $stime -eq $etime ]; then
	LogRecord "ERROR: the start and the end time are identical."
	ClearfAndExit 1
fi
if [ $stime -gt $etime ]; then
	etime=`$BIN/echo $etime |
	$AWK '{ printf"%d\n",'"$ONEDAYSECONDS"' + $1 }'`
	SAMPLING=OVERTHEDAY
else
	SAMPLING=INTHEDAY
fi
COUNT=`$BIN/echo $INTERVAL |
$AWK '{ printf"%d\n",('"$etime"'-'"$stime"')/$1 }'`
if [ $COUNT -eq 0 ]; then
	LogRecord "ERROR: the time parameters are inconsistent."
	ClearfAndExit 1
fi
}


#
# Test the sadc process is running for start process.
#
SadcExecTest() {
SADCCHECK=NO
SADCCHECKCOUNT=5
N=1
while [ $SADCCHECK != "YES" ]
do
	$BIN/ps -efl |
	$AWK '{
	if (( $15 == "/usr/lib/sa/sadc" ) || ( $15 == "/usr/bin/sar" ) || ( $15 == "sadc" )) {	# COR-005
		found="YES"
		sadcpid=$4						# COE-005
		}
	}
	END {	if ( found != "YES" )
			printf"NOTFOUND\n"
		else
			printf("%d\n",sadcpid)				# COR-005
	}' | $SORT -u > ${SARDODIR}/sadcpidlist
	for i in `$BIN/cat ${SARDODIR}/sadcpidlist | $BIN/grep [0123456789]`	# COR-009
	do
		SADCPID=$i
		if [ $SADCPID != "NOTFOUND" ] || [ $SADCPID -eq 0 ]; then
			RETVAL=`$AWK '{
			if ( $1 == '"$SADCPID"' ) {
				found="YES"
				exit
				}
			}
			END {	if ( found == "YES" )
					printf"OK\n"
				else
					printf"NG\n"
			}' ${SARDODIR}/pidlist`
		fi
		if [ $RETVAL = "OK" ]; then
			SADCCHECK=YES
			break
		fi
	done
	N=`$USRBIN/expr $N + 1`
	if [ $N -gt $SADCCHECKCOUNT ]; then
		RETVAL=NG
		SADCCHECK=YES
	else
		$BIN/sleep 1
	fi
done
if [ -f ${SARDODIR}/sadcpidlist ]; then
	$BIN/rm ${SARDODIR}/sadcpidlist
fi
}


#
# The sardo.sh termination process.
#
SarDoTermProc() {
SADCPID=$1
SEQUENCE=START
while [ -f ${SARDODIR}/go ]
do
	LogRecord "INFO:  wait for the end time [ $ETIME ] to finish."
	if [ $SAMPLING = "INTHEDAY" ]; then
		REMAIN=`$USRBIN/expr $etime - $ctime`
	else
		REMAIN=`$BIN/echo $ctime | $AWK '{
		todaysec = '"$ONEDAYSECONDS"' - $1
		nextdaysec = '"$etime"' - '"$ONEDAYSECONDS"'
		sec = todaysec + nextdaysec }
		END { printf"%d\n",sec }'`
	fi
	if [ $stime -gt $ctime ]; then
		waitsec=`$USRBIN/expr $stime - $ctime`
	else
		waitsec=0
	fi
	if [ $SEQUENCE = "START" ]; then
		REMAIN=`$USRBIN/expr $REMAIN - $waitsec`
	else
		REMAIN=$BASEINTERVAL
	fi
	BATCHMODE=YES
	LogRecord "INFO:  remaining time is $REMAIN seconds."

	$BIN/sleep $REMAIN

	SEQUENCE=STOP
	SADC=`$BIN/ps -efl |
	$AWK '{
	if (( $15 == "/usr/lib/sa/sadc" ) && ( $4 == '"$SADCPID"' )) {
		found="YES"
		exit
		}
	}
	END {	if ( found == "YES" )
			printf"ALIVE\n"
		else
			printf"NOTALIVE\n"
	}'`
	if [ $SADC = "NOTALIVE" ]; then
		$SARDOEXECSHELL stop
		cd ${LOGDIR}
		while [ -f $SARDODIR/pidlist ]
		do
			$BIN/sleep 1
		done
		$SARDOEXECSHELL tar 1>/dev/null 2>&1
		ClearfAndExit
	fi
done 2>/dev/null &
PID=$!
RETVAL=$?
export PID RETVAL
$BIN/echo $PID > $SARDODIR/termprocpid
}


#
# spautoexec: start sardo automatically.
#
spautoexec() {
#
BATCHMODE=YES; export BATCHMODE
#
if [ -f ${SARDODIR}/pidlist ]; then
	LogRecord "ERROR: instance $INSTANCE has already been started."
	ClearfAndExit 1
fi
SetTimeParameters $1 $2 $3
export INTERVAL COUNT ctime CTIME stime STIME etime ETIME SAMPLING
$SARDOEXECSHELL setup $INTERVAL $STIME $ETIME
if [ $? -ne 0 ]; then
	LogRecord "ERROR: spautoexec setup function terminated."
	ClearfAndExit 1
fi
$SARDOEXECSHELL start
if [ $? -ne 0 ]; then
	LogRecord "ERROR: spautoexec start function terminated."
	ClearfAndExit 1
fi
}


#
# Search mount point.
#
SearchMountPoint() {
DIRECTORY=$1
if [ $GETINFO = "THISHOST" ]; then
	$BIN/df -k > $WORK
else
	$BIN/cp $ESDIR/sarstats_${ESHOST}_${ESID}/end/df_k_info $WORK
fi
MOUNTPOINT=NO
MOUNT=$DIRECTORY
while [ $MOUNTPOINT = "NO" ]
do
	RESULT=`$BIN/echo $MOUNT |
	$AWK '{ n=split($1,array,"/") }
	END {	if ( n > 2 )
			printf"%s\n","'"$MOUNT"'"
		else
			if ( n == 2 )
				printf"THIS\n"
			else
				printf"YES\n" }'`
	if [ $RESULT = "YES" ]; then
		MOUNTPOINT="YES"
		break
	else
		if [ $RESULT = "THIS" ]; then
			MOUNTPOINT="YES"
			break
		else
			MOUNT=$RESULT
		fi
	fi
	MOUNT=`$USRBIN/dirname $MOUNT`
done
if [ $GETINFO = "THISHOST" ]; then
	VFSTAB=$ETCDIR/fstab
else
	VFSTAB=$ESDIR/sarstats_${ESHOST}_${ESID}/misc/fstab
fi
MOUNTCOUNT=`$BIN/cat $VFSTAB | $BIN/grep -v "#" |
$USRBIN/wc -l | $AWK '{ printf"%d\n",$1 }'`
N=1
DIR=$MOUNT
MOUNTPOINT=NOTFOUND
while [ $N != $MOUNTCOUNT ]
do
	for i in `$BIN/cat $VFSTAB | $BIN/grep -v "#" |
		  $AWK '{ printf"%d\n",$2 }'`
	do
		RESULT=`
		$AWK '{
		if ( "'"$DIR"'" == $6 ) {
			found="YES"
			name=$6
			exit
			}
		}
		END {	if ( found == "YES" )
				printf"%s\n",name
			else
				printf"NOTFOUND\n"
		}' $WORK`
		if [ $RESULT != "NOTFOUND" ]; then
			MOUNTPOINT=$RESULT
			break
		fi
		N=`$USRBIN/expr $N + 1`
	done
	N=$MOUNTCOUNT
done
if [ $MOUNTPOINT = "NOTFOUND" ]; then
	MOUNTPOINT="/"
fi

AVAILBLOCK=`$AWK {'
if ( $6 == "'"$MOUNTPOINT"'" )
	printf"%d\n", $2 * 0.9 - $3
}' $WORK`
$BIN/rm $WORK
}


#
# Estimate how many data the sardo.sh can gather into the LOGDIR directory.
#
spestimate() {
ESHOST=$1
ESID=$2
ESDIR=$3
ARGS=$#
if [ $ARGS -ne 3 ] && [ $ARGS -ne 0 ] ; then
	$BIN/echo "ERROR: spestimate invalid param."
	exit 1
fi
if [ "${ESHOST}_x" = "_x" ]; then
	ESHOST=`$BIN/uname -n`
	GETINFO=THISHOST
else
	HOSTNAME=`$BIN/uname -n`
	if [ $HOSTNAME = $ESHOST ]; then
		GETINFO=THISHOST
	else
		GETINFO=FROMFILES
	fi
fi
if [ "${ESID}_x" = "_x" ]; then
	if [ "${SARDO}_x" = "_x" ]; then
		ESID=1
	else
		ESID=$SARDO
	fi
fi

ESID=`$BIN/echo $ESID | $AWK '{ printf"%03d\n",$1 }'`

if [ "${ESDIR}_x" = "_x" ]; then
	ESDIR=$LOGDIR
fi

if [ ! -d ${ESDIR} ]; then
	$BIN/echo "ERROR: directory ${ESDIR} was not found."
	exit 1
fi

if [ -d ${ESDIR}/sarstats_${ESHOST}_${ESID} ]; then
	$BIN/echo "ERROR: now sardo_${ESHOST}_${ESID} directory is active."
	exit 1
fi


cd $ESDIR

FILECOUNT=`$BIN/ls |
$BIN/grep sarstats_${ESHOST}_${ESID}_.............tar.gz |
$USRBIN/wc -l | $AWK '{ printf"%d\n",$1 }'`

ARCHIVEFILE=`$BIN/ls |
$BIN/grep sarstats_${ESHOST}_${ESID}_.............tar.gz`

if [ $FILECOUNT -gt 1 ]; then
	$BIN/echo "ERROR: several sardo archive files were found."
	exit 1
fi

if [ "${ARCHIVEFILE}_x" = "_x" ]; then
	$BIN/echo "ERROR: no any sardo archive files were found."
	exit 1
fi

ARCHIVESIZE=`$USRBIN/du -k $ARCHIVEFILE  | $AWK '{ printf"%d\n",$1 }'`

$BIN/gunzip -d $ARCHIVEFILE
if [ $? -ne 0 ]; then
	$BIN/echo "ERROR: gunzip command was failed."
	exit 1
fi

TARFILENAME=`$BIN/echo $ARCHIVEFILE | $AWK '{ split($0,array,".") }
END {	printf"%s.tar\n",array[1] }'`

TARFILESIZE=`$USRBIN/du -k $TARFILENAME | $AWK '{ printf"%d\n",$1 }'`

$BIN/tar -xvf $TARFILENAME 1>/dev/null 2>&1
if [ $? -ne 0 ]; then
	$BIN/echo "ERROR: tar command was failed."
	exit 1
fi

cd $ESDIR/sarstats_${ESHOST}_${ESID}

CHECKCOUNT=12
N=1
EXIST=YES
while [ $EXIST = "YES" ]
do
	if [ ! -f sardosize_info ]; then
		$BIN/echo -e "\c" > sardosize_info
		EXIST=NO
		break
	fi
	if [ $N -lt $CHECKCOUNT ]; then
		$BIN/sleep $BASEINTERVAL
	else
		$BIN/echo "ERROR: sardosize_info file exists."
		cd ..
		$BIN/gzip $TARFILENAME
		if [ $? -ne 0 ]; then
			$BIN/echo "ERROR: gzip command was failed."
			$BIN/rm $WORK
			exit 1
		fi
		$BIN/rm -rf sarstats_${ESHOST}_${ESID}
		$BIN/rm $WORK
		exit 1
	fi
	N=`$USRBIN/expr $N + 1`
done

$USRBIN/du -k ../sarstats_${ESHOST}_${ESID} > $WORK
if [ $? -ne 0 ]; then
	$BIN/echo "ERROR: du command was failed."
	$BIN/rm $WORK
	exit 1
fi

SARDODIRSIZE=`$USRBIN/tail -1 $WORK | $AWK '{ printf"%d\n",$1}'`

$BIN/grep "sarstats_${ESHOST}_${ESID}/pacct" $WORK 1>/dev/null 2>&1
if [ $? -eq 0 ]; then
	PACCOUNTSIZE=`$BIN/grep "sarstats_${ESHOST}_${ESID}/pacct" $WORK |
	$AWK '{ printf"%d\n",$1 }'`
else
	PACCOUNTSIZE=0
fi

for i in begin end
do
	ONETIMETOTALSIZE=`$BIN/grep $i $WORK |
	$AWK '{ n = '"$ONETIMETOTALSIZE"' + $1 } END { printf"%d\n",n }'`
done

MISCSIZE=`$BIN/grep misc $WORK |
$AWK 'BEGIN {
tag=sprintf("../sarstats_%s_%s/misc","'"$ESHOST"'","'"$ESID"'") }
{	if ( $2 == tag ) {
		size=$1
		exit
	}
} END {	printf "%d\n",size }'`

ONETIMETOTALSIZE=`$USRBIN/expr $ONETIMETOTALSIZE + $MISCSIZE`

DATADIRSIZE=`$BIN/grep data $WORK | $AWK '{ printf"%d\n",$1 }'`
SADCDIRSIZE=`$BIN/grep sadc $WORK | $AWK '{ printf"%d\n",$1 }'`

PMAPFILECOUNT=`$BIN/grep pmap $WORK 1>/dev/null 2>&1
		if [ $? -ne 0 ]; then
			$BIN/echo 0
		else
			if [ -d pmap ]; then
				$BIN/ls pmap |
				$BIN/grep -v ps_efly |
				$BIN/grep -v pmapinfo |
				$USRBIN/wc -l | $AWK '{ printf"%d\n",$1 }'
			else
				$BIN/echo 0
			fi
		fi`
if [ $PMAPFILECOUNT -ne 0 ]; then
	PMAPDIRSIZE=`$BIN/grep pmap $WORK |
	$BIN/grep -v ps_efly |
	$BIN/grep -v pmapinfo |
	$AWK '{ printf"%d\n",$1 }'`
else
	PMAPDIRSIZE=0
fi

INTERVAL=`$AWK '{
printf"%d\n",$1 }' sardo.param`

STIME=`$AWK '{ printf"%s\n",$2 }' sardo.param`
S2I $STIME
stime=$TIMEVALUE

ETIME=`$AWK '{ printf"%s\n",$3 }' sardo.param`
S2I $ETIME
etime=$TIMEVALUE

if [ $stime -gt $etime ]; then
	etime=`$USRBIN/expr $etime + $ONEDAYSECONDS`
fi

SAMPLINGTIME=`$USRBIN/expr $etime - $stime`
SAMPLINGCOUNT=`$USRBIN/expr $SAMPLINGTIME \/ $INTERVAL`

SearchMountPoint $ESDIR

$BIN/echo "\
hostname           : `$BIN/echo ${ESHOST}`
hostid             : `$AWK '{	if ( $1 == "hostid_" ) {
						getline
						id=$1
					}
				}
		      END { printf"%s\n",id }' misc/hostid_info`
date               : `$BIN/cat date.info`

unitvalue          : 1024 bytes (kiro-bytes) 1 block unit

archivefilecount   : $FILECOUNT
availableblock     : $AVAILBLOCK
datadirectorysize  : $DATADIRSIZE
logdirectory       : $ESDIR
onetimetotalsize   : $ONETIMETOTALSIZE
paccountsize       : $PACCOUNTSIZE
packedarchivesize  : $ARCHIVESIZE
pmapdirectorysize  : $PMAPDIRSIZE
pmapfilecount      : $PMAPFILECOUNT
sadcdirectorysize  : $SADCDIRSIZE
samplinginterval   : $INTERVAL
samplingtime       : $SAMPLINGTIME
samplingcount      : $SAMPLINGCOUNT
sardodirectorysize : $SARDODIRSIZE
tarfilesize        : $TARFILESIZE
time info          : $stime $STIME $etime $ETIME
" >> $WORK

ONETIMEUNIT=$ONETIMETOTALSIZE
if [ $PMAPDIRSIZE -eq 0 ]; then
	PMAPUNIT=0
else
	PMAPUNIT=`$BIN/echo $PMAPDIRSIZE |
	$AWK '{ printf"%.0f\n",$1/'"$PMAPFILECOUNT"' }'`
fi
DATAUNIT=`$BIN/echo $DATADIRSIZE |
$AWK '{ printf"%.0f\n",$1/'" $SAMPLINGCOUNT"' }'`
SADCCOUNT=`$BIN/echo $SAMPLINGTIME |
$AWK '{ printf"%.0f\n",$1/'"$BASEINTERVAL"' }'`
SADCUNIT=`$BIN/echo $SADCDIRSIZE |
$AWK '{ printf"%.0f\n",$1/'"$SADCCOUNT"' }'`
PACCTUNIT=$PACCOUNTSIZE

$BIN/echo "\
dataunit           : $DATAUNIT  X  $SAMPLINGCOUNT
onetimeunit        : $ONETIMEUNIT  X  1
pacctunit          : $PACCTUNIT  X  n
pmapunit           : $PMAPUNIT  X  n
sadcunit           : $SADCUNIT  X  $SADCCOUNT
" >> $WORK
 
$BIN/echo "$intervaltimelist" |
$AWK 'BEGIN {
printf"  sampling time     sampling interval (sec)\n"
header=sprintf("  sec  min hour ") }
{ header=sprintf("%9s %9s ",header,$1) }
END {	printf"%s\n",header
	printf"=====+====+===+===========+==========+==========+==========+==="
	printf"=======+==========+==========+==========\n" }' >> $WORK

for samplingtime in `$BIN/echo "$samplingtimelist"`
do
	$BIN/echo $samplingtime |
	$AWK '{	printf"%5.0f %4.0f %3.0f ",$1,$1/60,$1/3600 }' >> $WORK
	for intervaltime in `$BIN/echo "$intervaltimelist"`
	do
		DATACOUNT=`$BIN/echo "" |
		$AWK '{	count = '"$samplingtime"' / '"$intervaltime"'
				printf"%.0f\n",count }'`

		SADCCOUNT=`$BIN/echo "" |
		$AWK '{	count = '"$samplingtime"' / '"$BASEINTERVAL"'
				printf"%.0f\n",count }'`

		PACTCOUNT=`$BIN/echo "" |
		$AWK '{	count = '"$samplingtime"' / '"$SAMPLINGTIME"'
				printf"%.0f\n",count }'`

		if [ $PMAPFILECOUNT -ne 0 ]; then
			PMAPCOUNT=`$BIN/echo "" |
			$AWK '{	count = '"$samplingtime"' / '"$SAMPLINGTIME"'
					count = count * '"$PMAPFILECOUNT"'
					printf"%.0f\n",count }'`
		else
			PMAPCOUNT=0
		fi

		$BIN/echo $ONETIMEUNIT |
		$AWK 'BEGIN {	pmapcount='"$PMAPCOUNT"'
					pmapunit='"$PMAPUNIT"' }
		{
		if ( '"$PMAPCOUNT"' == 0 ) {
			pmapsize = 0
			}
		else {
			pmapsize =  pmapcount * pmapunit
		}
		datasize = '"$DATACOUNT"' * '"$DATAUNIT"'
		pactsize = '"$PACTCOUNT"' * '"$PACCTUNIT"'
		sadcsize = '"$SADCCOUNT"' * '"$SADCUNIT"'
		sardosize = $1 + datasize + sadcsize + pactsize + pmapsize
		printf"%d\n",sardosize >"sardosize_info"
		totalsize = sardosize * 2 + sardosize * ( '"$ARCHIVESIZE"' / '"$TARFILESIZE"' )
		printf" %10.0f",totalsize }' >> $WORK
	done
	$BIN/echo "" >> $WORK
done
$BIN/echo "" >> $WORK

ONEDAYMAXSIZE=`$AWK '{	if ( $1 == "86400" ) {
					printf"%d\n",$NF
					exit
					}
			}' $WORK`

HOLDARCHIVES=`$BIN/cat sardosize_info |
$AWK '{
printf"%d\n",( '"$AVAILBLOCK"' - '"$ONEDAYMAXSIZE"' ) / ( $1 * ( '"$ARCHIVESIZE"' / '"$TARFILESIZE"' )) }'`

CYCLE=`$BIN/echo $ONEDAYMAXSIZE |
$AWK '{ printf"%d\n",'"$AVAILBLOCK"' / $1 }'`

$BIN/cat sardosize_info | $AWK '{
datasize = '"$DATACOUNT"' * '"$DATAUNIT"'
pactsize = '"$PACTCOUNT"' * '"$PACCTUNIT"'
pmapsize = '"$PMAPCOUNT"' * '"$PMAPUNIT"'
sadcsize = '"$SADCCOUNT"' * '"$SADCUNIT"'
printf"the largest blocks :\n"
printf"datadirectory      : %d\n",datasize
printf"onetimedirectories : %d\n",'"$ONETIMEUNIT"'
printf"pacctdirectory     : %d\n",pactsize
printf"pmapdirectory      : %d\n",pmapsize
printf"sadcdirectory      : %d\n",sadcsize
printf"sardodirectory     : %d\n\n",'"$ONETIMEUNIT"'+pmapsize+datasize+sadcsize+pactsize
printf"packedarchivefile  : %d\n",$1 * ( '"$ARCHIVESIZE"' / '"$TARFILESIZE"' )
printf"\n"
}' >> $WORK

$BIN/echo "\
INFO:  $ESDIR can hold [ $HOLDARCHIVES ] packed tar archive files, 
INFO:  during the sardo is gathering the latest one of data.
INFO:  sardo can gather maximum for [ $CYCLE ] days data in the directory.

END REPORT" >> $WORK

$BIN/mv $WORK ${ESDIR}/sarstats_${ESHOST}_${ESID}/estimate

$BIN/rm sardosize_info
}


#
# spstart: start sardo.
#
spstart() {
EXECMODE=$1
if [ "${MODE}_x" = "_x" ]; then
	EXECMODE=NOAUTO
fi
if [ ! -f $PARAMETER ]; then
	LogRecord "ERROR: spstart instance $INSTANCE sardo.param wasn't found."
	ClearfAndExit 1
fi
if [ -f ${SARDODIR}/pidlist ]; then
	LogRecord "ERROR: instance $INSTANCE has already been started."
	ClearfAndExit 1
fi
$BIN/date '+%Y.%m.%d' > ${SARDODIR}/date.info
SECONDS=`$AWK '{ printf"%s\n",$1 }' $PARAMETER`
STARTTIME=`$AWK '{ printf"%s\n",$2 }' $PARAMETER`
STOPTIME=`$AWK '{ printf"%s\n",$3 }' $PARAMETER`
SetTimeParameters $SECONDS $STARTTIME $STOPTIME
export INTERVAL COUNT ctime CTIME stime STIME etime ETIME SAMPLING
LogRecord "INFO:  sampling interval  : $INTERVAL"
LogRecord "INFO:  sampling count     : $COUNT"
LogRecord "INFO:  current time       : $ctime $CTIME"
LogRecord "INFO:  start and end time : $stime $STIME $etime $ETIME"
LogRecord "INFO:  sampling mode      : $SAMPLING"
if [ $ctime -gt $etime ]; then
	LogRecord "ERROR: the current time greater than the end time."
	ClearfAndExit 1
fi
if [ $stime -gt $ctime ]; then
	waitsec=`$USRBIN/expr $stime - $ctime`
	LogRecord "INFO:  awaiting start $waitsec sec."
	$BIN/sleep $waitsec
else
	waitsec=0
fi
LogRecord "INFO:  sardo will be started with $INTERVAL sec interval."
# COR-021 start.
if [ "${PACCTMODE}_x" = "_x" ]; then
	PACCTMODE="RUN"
fi
export PACCTMODE
# COR-021 end.
CheckPacct
if [ $PACCTMODE = "RUN" ]; then
	if [ ! -d $SARDODIR/pacct ]; then
		$BIN/mkdir $SARDODIR/pacct
		if [ $? -ne 0 ]; then
			LogRecord "ERROR: process accounting directory couldn't be made."
			ClearfAndExit 1
		fi
	fi
	# COR-021 start.
	if [ ! -e $USRSBIN/accton ]; then
		$BIN/echo "NOTE:  accton command not found."
		PACCTMODE="DONTRUN"
	else
		$TOUCH $SARDODIR/pacct/pacct_info
		$USRSBIN/accton $SARDODIR/pacct/pacct_info
		if [ $? -ne 0 ]; then
			LogRecord "ERROR: starting process account was failed."
			ClearfAndExit 1
		else
			$BIN/echo $$ > ${SARDODIR}/pacctpid
			LogRecord "INFO:  process accounting has been started."
		fi
	fi
	# COR-021 end.
	$TOUCH $ACCTLOCK
fi
$BIN/echo "$ONETIMECMD" > $TEMPFILE	; DoCommands begin
$BIN/echo "$DURCMD"     > $TEMPFILE	; DoDurationCmds data
$BIN/echo "$NDCMD"      > $TEMPFILE	; DoNonDurationCmds data
$BIN/echo "$SADCCMD"    > $TEMPFILE	; DoSaDcCommand sadc
whichcmd $EVENTCHECK
if [ $COMMANDPATH != "NOTFOUND" ]; then
	if [ -x $COMMANDPATH ]; then
		while [ -f $SARDODIR/go ]
		do
			$COMMANDPATH &					# COR-003
			$BIN/sleep $INTERVAL
		done &
		# COR-003 start.
		RETVAL=$?
		EVENTPID=$!
		EVENTCMD=@path
	else
		EVENTCMD=NONE
		# COR-003 end.
		$BIN/echo $PID >> $SARDODIR/pidlist
	fi
else
	if [ -f ./$EVENTCHECK ]; then
		if [ -x ./$EVENTCHECK ]; then
			while [ -f $SARDODIR/go ]
			do
				./$EVENTCHECK &				# COR-003
				$BIN/sleep $INTERVAL
			done &
		# COR-003 start.
			RETVAL=$?
			EVENTPID=$!
			EVENTCMD=@pwd
		else
			EVENTCMD=NONE
		fi
	else
		EVENTCMD=NONE
		# COR-003 end.
	fi
fi

# COR-003 start.
if [ $EVENTCMD != "NONE" ]; then
	if [ $RETVAL -ne 0 ]; then
		LogRecord "NOTE:  $EVENTCHECK $EVENTCMD couldn't be running."
	else
		LogRecord "INFO:  $EVENTCHECK $EVENTCMD was successfully scheduled."
		$BIN/echo $EVENTPID >> $SARDODIR/pidlist
	fi
fi
# COR-003 end.

SadcExecTest
if [ $RETVAL != "OK" ]; then
	LogRecord "ERROR: sadc process pid [ $SADCPID ] wasn't running."
	$BIN/echo "ERROR: sadc process pid [ $SADCPID ] wasn't running."
	ClearfAndExit 1
fi
SarDoTermProc $SADCPID
if [ $RETVAL -ne 0 ]; then
	LogRecord "ERROR: the termination process starting was failed."
	ClearfAndExit 1
else
	LogRecord "INFO:  termproc pid [ $PID ] was started successfully."
fi
}


#
# spstop: stop sardo.
#
spstop() {
PID=$$
arg1=$1
if [ "${arg1}_x" = "_x" ]; then
	MODE=NORMAL
else
	if [ $arg1 = "force" ]; then
		MODE=FORCE
	else
		LogRecord "NOTE:  spstop invalid mode."
		MODE=NORMAL
	fi
fi
if [ -f $SARDODIR/go ]; then
	$BIN/rm $SARDODIR/go
fi
if [ -f $SARDODIR/termprocpid ] && [ $MODE = "FORCE" ] ; then
	pid=`$BIN/cat $SARDODIR/termprocpid`
	ppid=`$BIN/ps -efl |
	$AWK '{
	if ( $5 == '"$pid"' ) {
		n=split($15,array,"/")
		if ( array[n] == "sleep")
			found="YES"
		processid=$4
		exit
		}
	} END {	if (( found == "YES" ) && ( processid != 1 ))
			printf"%d\n",processid
		else
			printf"NOPPID\n"
	}'`
	$BIN/kill -9 $pid 1>/dev/null 2>&1
	$BIN/kill -9 $ppid 1>/dev/null 2>&1
	$BIN/rm $SARDODIR/termprocpid
else
	if [ -f $SARDODIR/termprocpid ]; then
		termprocpid=`$BIN/cat $SARDODIR/termprocpid`
		$BIN/rm $SARDODIR/termprocpid
	fi
fi
if [ -f $SARDODIR/pidlist ]; then
	for pid in `$BIN/cat $SARDODIR/pidlist`
	do
		ppid=`$BIN/ps -efl |
		$AWK '{
		if ( $5 == '"$pid"' ) {
			n=split($15,array,"/")
			if ( array[n] == "sleep")
				found="YES"
			processid=$4
			exit
			}
		} END {	if ( found == "YES" )
				printf"%d\n",processid
			else
				printf"NOPPID\n"
		}'`
		$BIN/kill -9 $pid 1>/dev/null 2>&1
		if [ $ppid != "NOPPID" ]; then
			kill -9 $ppid 1>/dev/null 2>&1
		fi
		$BIN/sleep 1
	done
	BlankLine
fi
$BIN/echo "$ONETIMECMD" > $TEMPFILE	; DoCommands end
if [ -f ${SARDODIR}/pacctpid ]; then
	# COR-014 start.
	ACCTLATESTVERSION=`$USRSBIN/accton -V |
	$AWK 'BEGIN {	acctlv="'"$acctlv"'" }
	{
	n=split($1,command,"/")
	if (( command[n] == "accton:" ) && ( $3 == "Accounting" )) {
		split($6,release,")")
		n=split(release[1],array,".")
		for ( i = 0; i < n; i++ )
			sysid=int(sprintf("%02d%02d",sysid,array[i]))	# COR-019
		n=split(acctlv,array,".")
		for ( i = 0; i < n; i++ )
			chkid=int(sprintf("%02d%02d",chkid,array[i]))	# COR-019
		if ( sysid == chkid ) {
			result="EQUAL"
			exit
			}
		else {
			if ( sysid > chkid ) {
				result="HIGH"
				exit
				}
			else {
				result="LOW"
				exit
				}
			}
		}
	} END {	printf"%s\n",result }'`
	# COR-012 start.
	if [ $KERNEL = "Debian" ] || [ $ACCTLATESTVERSION = "EQUAL" ]; then
	# COR-014 end.
		$USRSBIN/accton off
	else
		# COR-020 start.
		ACCTVERSION=`$USRSBIN/accton -V |
		$AWK 'BEGIN { tag="NO" }
		{
		if (( $2 == "GNU" ) && ( $3 == "Accounting" ) && ( $4 == "Utilities" )) {
			split($6,release,")")
			tag=release[1]
			}
		} END { printf("%s\n",tag)
		}'`
		if [ $ACCTVERSION = "$acctcent7" ] || [ $ACCTVERSION = "$acctcent8" ] || [ $ACCTVERSION = $acctlatest ]; then # COR-026
			$USRSBIN/accton off
		else
			$USRSBIN/accton
		fi
		# COR-020 end.
	fi
	# COR-012 end.
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: stop process accounting was failed."
		ClearfAndExit 1
	else
		LogRecord "INFO:  process accounting was stopped."
	fi
	$BIN/rm $ACCTLOCK
	$BIN/rm ${SARDODIR}/pacctpid
else
	LogRecord "INFO:  process accounting wasn't stopped."
fi
if [ ! -d $SARDODIR/misc ]; then
	$BIN/mkdir $SARDODIR/misc
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: making misc directory was failed."
		ClearfAndExit 1
	fi
fi

cd $SARDODIR/misc

for i in `$BIN/echo "$GATHERFILES"`
do
	case $i in
	$PROC/net|$PROC/sys|$PROC/sysvipc|$PROC/tty)
		$BIN/cp -a $i . 2>/dev/null
		$BIN/chmod ugo+r .		;;		       # COR-023
	*)	if [ -f $i ]; then
			$BIN/cp $i .
			# COR-022 start.
			if [ $i = "$USERBIN/sar" ]; then
				chmod 755 $i
			fi
			# COR-022 end.
			$BIN/chmod ugo+r .			       # COR-023
			if [ $? -ne 0 ]; then
				# COR-013 start.
				$BIN/uname -r | $BIN/grep gentoo >/dev/null
				if [ $? -eq 0 ] && [ $i = "$PROC/ioports" ]; then
					LogRecord "NOTE:  file $i copy was failed."
				else
					LogRecord "ERROR: file $i copy was failed."
					ClearfAndExit 1
				fi
				# COR-013 end.
			fi
		fi				;;
	esac
done
cd $SARDODIR
for i in `$USRBIN/find * -print | $BIN/grep -v tempfile`
do
	if [ -f $i ]; then
		$USRBIN/wc -c $i
		if [ $? -ne 0 ]; then
			LogRecord "ERROR: wc command was failed."
			ClearfAndExit 1
		fi
	fi
done > $TEMPFILE
$BIN/mv $TEMPFILE $SARDODIR/misc/fsize_info
if [ $? -ne 0 ]; then
	LogRecord "ERROR: moving fsize_info file was failed."
	ClearfAndExit 1
fi
$BIN/echo "$MISCCMD" > $TEMPFILE	; DoCommands misc
$BIN/rm $SARDODIR/pidlist
}


#
# Archive files.
#
sparchive() {
if [ -f $SARDODIR/pidlist ]; then
	LogRecord "ERROR: sardo instance $INSTANCE is still running."
	ClearfAndExit 1
fi
cd $LOGDIR
DIR=`$BASENAMECMD $SARDODIR`
DATE=`$BIN/date '+%y%m%d%H%M%S'`
$BIN/tar -cvf ${SARDODIR}_${DATE}.tar $DIR
if [ $? -ne 0 ]; then
	LogRecord "ERROR: tar command was failed."
	ClearfAndExit 1
fi
$BIN/gzip ${SARDODIR}_${DATE}.tar
if [ $? -ne 0 ]; then
	LogRecord "ERROR: gzip command was failed."
	ClearfAndExit 1
fi
ROOTDIR=/
TESTDIR=$EXECDIRECTORY
FOUND=NO
while [ "$ROOTDIR" != "$TESTDIR" ]
do
	TESTDIR=`$USRBIN/dirname $TESTDIR`
	if [ $TESTDIR = $LOGDIR ]; then
		FOUND=YES
		ROOTDIR=/
	fi
done
if [ $FOUND = "YES" ]; then
	$BIN/echo "NOTE:  Although tar archive and compress were successfully completed,"
	$BIN/echo "NOTE:  the working directory $EXECDIRECTORY is below $LOGDIR."
	$BIN/echo "NOTE:  $DIR directory won't be cleared."
else
	$BIN/rm -rf $SARDODIR
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: remove $SARDODIR was failed."
		ClearfAndExit 1
	fi
	$BIN/echo "INFO:  tar archive and compress were successfully completed."
fi
}


#
# Collect pmap -x information.
#
spexpmap() {
if [ ! -d $SARDODIR/pmap ]; then
	$BIN/mkdir $SARDODIR/pmap
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: making pmap directory was failed."
		ClearfAndExit 1
	fi
fi
DATE=`$BIN/date '+%y%m%d%H%M%S'`
$BIN/ps -eflyww > $SARDODIR/pmap/ps_eflyww_${DATE}_info	# COR-016 COR-018
for pid in `$BIN/ps -efl |
$AWK '{	if ( $4 != "PID" )
			print $4 }'`
do
	$BIN/echo "bgn: $pid"
	$BIN/cat ${PROC}/${pid}/maps 2>/dev/null
	$BIN/echo "end: $pid"
done > $SARDODIR/pmap/pmap_x_${DATE}_info
#if [ $? -ne 0 ]; then
#	LogRecord "ERROR: pmap -x was failed."
#	ClearfAndExit 1
#fi
LogRecord "INFO:  an intermediate pmap -x command was completed."
}

#
# COR-011 start.
#
# Collect /proc/meminfo information.
#
spexmemstat() {
if [ ! -d $SARDODIR/memstat ]; then
	$BIN/mkdir $SARDODIR/memstat
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: making memstat directory was failed."
		ClearfAndExit 1
	fi
fi
DATE=`$BIN/date '+%y%m%d%H%M%S'`
$BIN/cat $PROC/meminfo > $SARDODIR/memstat/memstat_${DATE}_info
RETVAL=$?
if [ $RETVAL -ne 0 ]; then
	LogRecord "ERROR: an intermediate meminfo copy was failed."
	ClearfAndExit 1
else
	LogRecord "INFO:  an intermediate meminfo copy was completed."
fi
}
# COR-011 end.


#
# Unpack sarstats data into the sardo directory.
#
spunpack () {
WORK=$LOGDIR/workfile$$
cd $LOGDIR
$BIN/ls  sarstats_*_${INSTANCE}_*.tar.gz 2>/dev/null > $WORK
if [ ! -s $WORK ]; then
	$BIN/echo "ERROR: no any unpack files were found."
	ClearfAndExit 1
fi
N=`$USRBIN/wc -l $WORK | $AWK '{ print $1 }'`
if [ $N -gt 1 ]; then
	if [ $BATCHMODE = "YES" ]; then
		$BIN/echo "NOTE:  any unpack files list couldn't be displayed."
		ClearfAndExit 1
	fi
	input="NO"
	while [ $input = "NO" ]
	do
		BlankLine
		$BIN/ls -l sarstats_*_${INSTANCE}_*.tar.gz | $AWK '{
		printf"%s  %3s %2s %s %9d\n",$9,$6,$7,$8,$5}' > $WORK
		$AWK '{ printf"%10d.  %s\n",NR,$0 }' $WORK
		$BIN/echo -e "         Several unpack files found.\n"
		$BIN/echo -n "         Select file number ? : "
		read INPUT
		if [ "${INPUT}_x" != "_x" ]; then
			if [ ${INPUT} -le ${N} ] && [ ${INPUT} != 0 ]; then
				input="YES"
				FILENAME=`$AWK '
				{	if ( NR == "'$INPUT'" )
						printf"%s\n",$1 }' $WORK`
			fi
		else
			input="NO"
		fi
	done
	$BIN/echo "INFO:  the unpack file $FILENAME was chosen."
else
	FILENAME=`$BIN/cat $WORK`
	$BIN/echo "NOTE:  the unpack file $FILENAME was assumed."
fi
if [ -s $WORK ]; then
	$BIN/rm $WORK
fi
FILENAME=`$BIN/echo $FILENAME |
$AWK '{	split($1,array,".")
		printf"%s.tar\n",array[1],array[2] }'`
DIRECTORYNAME=`$BIN/echo $FILENAME |
$AWK '{	split($1,array,".")
		split(array[1],dir,"_")
		printf"%s_%s_%s\n",dir[1],dir[2],dir[3] }'`
if [ -d $DIRECTORYNAME ]; then
	$BIN/echo "ERROR: directory $DIRECTORYNAME already exists."
	ClearfAndExit 1
fi
$BIN/gunzip -d ${FILENAME} 2>/dev/null
if [ $? -ne 0 ]; then
	$BIN/echo "ERROR: gunzip command was failed."
	ClearfAndExit 1
fi
$BIN/tar -xvf $FILENAME
if [ $? -ne 0 ]; then
	$BIN/echo "ERROR: tar -xvf command was failed."
	ClearfAndExit 1
fi
$BIN/echo "INFO:  spunpack was successfully completed."
}


############################################
#         Main routine start here.         #
############################################

#
# Set arguments.
#
EXECSHELL=$0
numberofargs=$#

# COR-012 start.
if [ -f /usr/bin/awk ] && [ ! -f /bin/awk ]; then
	AWK=/usr/bin/awk
	BASENAMECMD=/usr/bin/basename
	SORT=/usr/bin/sort
	TOUCH=/usr/bin/touch
	ACCTLOCK=/var/lock/psacct
	KERNEL=Debian
else
	AWK=/bin/awk
	BASENAMECMD=/bin/basename
	SORT=/bin/sort
	TOUCH=/bin/touch
	ACCTLOCK=/var/lock/subsys/psacct
	KERNEL=RedHat
fi
# COR-012 end.

execshell=`$BASENAMECMD $EXECSHELL`
FUNCTION=$1; ARG2=$2; ARG3=$3; ARG4=$4
LANG=C; export LANG

if [ $numberofargs = 0 ]; then
	$BIN/echo "ERROR: argument missing."
	exit 1
fi

#
# Determine the LOGDIR environment variable.
#
if [ "${LOGDIR}_x" = "_x" ]; then
	LOGDIR=/var/tmp
fi
	

#
# Get and check the OS name.
#
OS=`$BIN/uname -s`
if [ $OS != $OSNAME ]; then
	$BIN/echo "ERROR: $OS is not $OSNAME."
	exit 1
fi


#
# Get the working directory and the actual sardo.sh path name.
#
EXECDIRECTORY=`$BIN/pwd`
whichcmd $execshell
RESULT=$COMMANDPATH
if [ $RESULT = "NOTFOUND" ]; then
	ISDOT=`$BIN/echo $EXECSHELL | $AWK '{
	if ( substr($1,1,1) == "." )
		printf"YES\n"
	else
		printf"NO\n"
	}'`
	if [ $ISDOT = "YES" ]; then
		SARDOEXECSHELL=${EXECDIRECTORY}/${execshell}
	else
		SARDOEXECSHELL=$EXECSHELL
	fi
else
	SARDOEXECSHELL=$COMMANDPATH
fi

# COR-003 start.
EXEC=`$USRBIN/dirname $SARDOEXECSHELL`
export EXEC
# COR-003 end.

#
# Check root user uid.
#
UidCheck

#
# Set INSTANCE and the other environment variables.
#
SetInstance


#
# Set BATCHMODE environment variable.
#
SetBatchMode

#
# Check and make SARDO directory under the workdirectory $LOGDIR.
#
SarDoDirectory

#
# making up version_info file
#
CheckToolsLevel

#
# Go to the individual function.
#
case $FUNCTION in
auto)		spautoexec	$ARG2 $ARG3 $ARG4	;;
estimate)	spestimate	$ARG2 $ARG3 $ARG4	;;
memstat)        spexmemstat                             ;;              #COR-011
pmap)		spexpmap				;;
setup)		spsetup		$ARG2 $ARG3 $ARG4	;;
start)		spstart					;;
stop)		spstop		$ARG2			;;
tar)		sparchive				;;
unpack)		spunpack				;;
*)		LogRecord "ERROR: argument missing. [ $FUNCTION ]"
		ClearfAndExit 1				;;
esac

#
# clear work files and exit.
#
ClearfAndExit

#
# end script.
